/*
PFEdemo1 - Probas

Copyright (C) 2002 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import java.awt.Toolkit;

import simu.*;

//utilise Proba et MoreProba

/**
 * Algorithme de simulation pour SimuLab.
 * Poisson
 */

//	compilation: javac -target 1.1 -bootclasspath ~/Java/jdk1.1/JDKClasses.zip -extdirs "" -encoding ISO-8859-1 -classpath .:SimuLab.jar Erlang.java

public class Erlang extends SimuApplet {
	final static int nipa = 1; // nombre d'itrations par affichage
	final static int npts = 128;
	Affichage affPlotLive;
	double lambda1, nu;
	double lambda;
	int k;
	boolean distrib = true;
	boolean drawfn = true;
	boolean tcl = true;
	long ct;
	int nt;
	double xmin, xmax, ymin, ymax;
	int tshow;
	double[] sample;
	double[] tx;
	double[] tft;
	double[] tfg;
	double[] tdt;
	double[] tfn;
	int is;

    public Erlang() {
        // paramtres d'entre
        ParamIn lambdaParam = new ParamIn();
        lambdaParam.type = "nombre";
        lambdaParam.label = "lambda";
        lambdaParam.titre = "lambda";
        lambdaParam.acquisition = "champ";
        lambdaParam.defaut = "2.e-4";
        addParamIn(lambdaParam);
        
        ParamIn nuParam = new ParamIn();
        nuParam.type = "nombre";
        nuParam.label = "nu";
        nuParam.titre = "nu";
        nuParam.acquisition = "champ";
        nuParam.defaut = "4.0";
        addParamIn(nuParam);
        
        ParamIn tclParam = new ParamIn();
        tclParam.type = "string";
        tclParam.label = "tcl";
        tclParam.titre = "Gauss limit";
        tclParam.acquisition = "case";
        tclParam.defaut = "on";
        addParamIn(tclParam);
        
        ParamIn demoParam = new ParamIn();
        demoParam.type = "string";
        demoParam.label = "demo";
        demoParam.titre = "flux";
        demoParam.acquisition = "choix";
		String[] lchoix = {"poisson (nu = 1)", "erlang", "erlang*nu"};
		demoParam.choix = lchoix;
        demoParam.defaut = "poisson (nu = 1)";
        addParamIn(demoParam);
        
        // paramtres de sortie
		
		// distribution
        ParamOut dx = new ParamOut();
        dx.type = "nombre";
        dx.label = "dx";
        dx.titre = "";
        addParamOut(dx);
        ParamOut dy = new ParamOut();
        dy.type = "nombre";
        dy.label = "dy";
        dy.titre = "";
        addParamOut(dy);
		
		// fct de rpartition
        ParamOut fx = new ParamOut();
        fx.type = "nombre";
        fx.label = "fx";
        addParamOut(fx);
        ParamOut fy = new ParamOut();
        fy.type = "nombre";
        fy.label = "fy";
        addParamOut(fy);
        
		// Gauss limit
        ParamOut gx = new ParamOut();
        gx.type = "nombre";
        gx.label = "gx";
        addParamOut(gx);
        ParamOut gy = new ParamOut();
        gy.type = "nombre";
        gy.label  = "gy";
        gy.titre = "";
        addParamOut(gy);
        
		// point de changement
        ParamOut px = new ParamOut();
        px.type = "nombre";
        px.label = "px";
        addParamOut(px);
        ParamOut py = new ParamOut();
        py.type = "nombre";
        py.label  = "py";
        py.titre = "";
        addParamOut(py);
        
		// infos (n itrations)
        ParamOut info = new ParamOut();
        info.type = "string";
        info.label = "info";
        addParamOut(info);
		
        // options d'affichage
		appletDoc("Erlang_doc.xml");
        
        // affichage "plotlive"
        affPlotLive = new Affichage();
        affPlotLive.type = "plotlive";
        affPlotLive.titre = "flux d'Erlang";
        String[][] pparams = {{"dx", "dy"},{"fx","fy"},{"gx","gy"},{"px","py"}};
        affPlotLive.params = pparams;
		affPlotLive.points = "none";
		affPlotLive.connecter = true;
		affPlotLive.effacer = true;
        addAffichage(affPlotLive);
    }
	
    // calcul live (utilis avec affichage "plotlive")
    // initialisation
    public void initCalculLive(ListeValeurs in) throws SimuException {
		final double tmax = 1.0e7;
		
        lambda1 = in.lireDouble("lambda");
        nu = in.lireDouble("nu");
		tcl = "on".equals(in.lireString("tcl"));
        String demo = in.lireString("demo");
		
		if ("poisson (nu = 1)".equals(demo)) {
			lambda = lambda1;
			k = 1;
		} else if ("erlang".equals(demo)) {
			lambda = lambda1;
			k = (int)Math.round(nu);
		} else if ("erlang*nu".equals(demo)) {
			lambda = lambda1*Math.round(nu);
			k = (int)Math.round(nu);
		}
		double[] boxF = MoreProba.bestBox2(lambda, k);
		xmin = boxF[0];
		xmax = boxF[1];
		ymin = boxF[2];
		ymax = boxF[3];
		
		affPlotLive.xdebut = "" + xmin;
		affPlotLive.xfin = "" + xmax;
		affPlotLive.ydebut = "" + ymin;
		affPlotLive.yfin = "" + ymax;
		
		tx = new double[npts];
		tft = new double[npts];
		tfg = new double[npts];
		
		// Prepare theoretical distribution function.
		for (int i=0; i<npts; i++)
			tx[i] = xmin + i*(xmax-xmin)/(npts-1.0);
		tft = Proba.fgamma(k, lambda, tx);
		
		// Prepare Gaussian distribution function.
		if (tcl)
			tfg = Proba.fnormal(k/lambda, Math.sqrt(k)/lambda, tx);
		
		// Generate the Poisson sample size.
		int[] num = Proba.sPoisson(lambda*tmax, 1);
		
		// Allocate space for Poisson time sample.
		sample = new double[num[0]+2];
		// and create it:
		for (int i=1; i<sample.length-1; i++)
			sample[i] = Math.random();
		// Set first one to time origin.
		sample[0] = 0.0;
		// And last one over time limit.
		sample[sample.length-1] = 2.0;
		Proba.hpsort(sample);
		for (int i=0; i<sample.length; i++)
			sample[i] = sample[i]*tmax;
		
		// Generate k-Erlang Flux
		int ssize = (int)Math.floor(num[0]/(double)k);
		for (int i=1; i<=ssize; i++) //  <----- et sample[0] ?
			sample[i] = sample[k*i];
		
		// Set last one over time limit.
		sample[ssize+1] = tmax + 1;
		
		// Allocate space for empirical repartition function.
		tdt = new double[ssize];
		tfn = new double[ssize];
		
		// Computes Time differences.
		for (int i=0; i<ssize; i++)
			tdt[i] = sample[i+1] - sample[i];
		
		// Choose time of visual simulation.
		tshow = (int)(0.1*tmax);
		int rank = Proba.rLocate(tshow, sample, ssize+1);
		int sshow = 0 + rank - 1;
		
		nt = 0;
		is = 0;
		ct = System.currentTimeMillis();
    }
    
    // Calcul du temps t au temps t+1.
    public ListeValeurs calculLive() throws SimuException {
        if (nt >= tshow)
            return null; // fin du calcul
		
		double xx=0;
		int tempo = 0;
		for (int ii=0; ii<nipa; ii++) {
		
			if (is == sample.length - 1)
				return(null);
			
			int nt2 = (int)Math.ceil(sample[is+1]);
			tempo += nt2 - nt;
			nt = nt2;
			
			if (drawfn) {
				// Save event.
				xx = tdt[is];
				// Compute empirical Dt distribution function.
				Proba.rMerge1(tdt, is+1);
				for (int j=0; j<is+1; j++)
					tfn[j] = (j+1.0)/(is+1);
			}
			is++;
		}
		
		tempo = tempo/5;
		
		//System.out.println("tempo="+tempo);
		// pour bloquer le programme jusqu' ct+tempo ms
		long ct2 = System.currentTimeMillis();
		if (ct2 - ct < tempo) {
	        try {
	            Thread.currentThread().sleep(tempo - (ct2 - ct));
	        } catch (InterruptedException e) {}
			ct = ct + tempo;
		} else
			ct = ct2;
		Toolkit.getDefaultToolkit().beep();
		
        ListeValeurs out = new ListeValeurs();
		
		// on encadre le point
		// rsum en fortran par Call PgPoint(1,xx,Fn(Minloc(Abs(Dt(:i)-xx))),CIRCLE)
		double miny = 1e50;
		int minloc = 0;
		for (int i=0; i<is; i++)
			if (Math.abs(tdt[i] - xx) < miny) {
				miny = Math.abs(tdt[i] - xx);
				minloc = i;
			}
		double py = tfn[minloc];
		double dpx = (xmax - xmin)/80;
		double dpy = (ymax - ymin)/80;
		out.ajouterDouble("px", xx - dpx);
		out.ajouterDouble("py", py - dpy);
		out.ajouterDouble("px", xx + dpx);
		out.ajouterDouble("py", py - dpy);
		out.ajouterDouble("px", xx + dpx);
		out.ajouterDouble("py", py + dpy);
		out.ajouterDouble("px", xx - dpx);
		out.ajouterDouble("py", py + dpy);
		out.ajouterDouble("px", xx - dpx);
		out.ajouterDouble("py", py - dpy);
		
		out.ajouterDouble("fx", xmin);
		out.ajouterDouble("fy", 0);
		if (drawfn) {
			out.ajouterDouble("fx", tdt[0]);
			out.ajouterDouble("fy", 0);
			for (int i=0; i<is; i++) {
				//System.out.println("tdt["+i+"]="+tdt[i]+" tfn["+i+"]="+tfn[i]);
				if (i == is - 1 || tdt[i+1] > tdt[i]) {
					out.ajouterDouble("fx", tdt[i]);
					out.ajouterDouble("fy", tfn[i]);
				}
				
				if (i != is - 1 && tdt[i+1] > tdt[i]) {
					out.ajouterDouble("fx", tdt[i+1]);
					out.ajouterDouble("fy", tfn[i]);
				}
			}
			out.ajouterDouble("fx", xmax);
			out.ajouterDouble("fy", tfn[is-1]);
		}
		
		if (distrib) {
			for (int i=0; i<npts; i++) {
				out.ajouterDouble("dx", tx[i]);
				out.ajouterDouble("dy", tft[i]);
			}
		}
		if (tcl) {
			for (int i=0; i<npts; i++) {
				out.ajouterDouble("gx", tx[i]);
				out.ajouterDouble("gy", tfg[i]);
			}
		} else { // pb affichage: comment ne pas avoir  mettre de valeur ici ?
			out.ajouterDouble("gx", 0);
			out.ajouterDouble("gy", 0);
		}
		
		out.ajouterString("info", "nb de points: " + is + "  dernire tempo: " + tempo);
		
        return out;
    }
}
